#  "LoadUtil.rb"
#
#  An example / template by Dan Rathbun.
#
#  VERSION : 2.0.0
#
#  A mixin module to create a private "loaded_files" array, and
#  getter and setter methods, an author can mix into, their own
#    nested class and module namespaces.
#
#  This file must reside within an Author's sub-directory.
#
#  Use an editor, to search and replace all "Author" occurances
#    with YOUR toplevel author / company namespace identifier.
#    Do not search on "whole word only", search also as a
#    substring so "Author" gets replaced in all instances.
#  You can then delete this instruction, change the whole header.
#
#  USAGE:
#
#   module Author
#
#     require("author/lib/loadutil")
# 
#     class Widget
#     
#       include(Author::LoadUtil)
#       file_loaded('Author::Widget')
#
#     end
#     
#   end # Author toplevel namespace


module Author

  module LoadUtil
  
    #  CONSTANTS
    
    VERSION = '2.0.0'

    #{#  CLASS/MODULE VARIABLES
    #
      @@loaded_files = [] unless defined?(@@loaded_files)
      @@included = [] unless defined?(@@included)
    #
    #}#
    
    #{#  PRIVATE CLASS/MODULE RUBY INTERNAL CALLBACKS
    #
      private

      def self.append_features(mod)
        unless mod.class==(Class) || mod.class==(Module)
          # Do not allow instance objects to be passed as an argument.
          raise(TypeError,"Module or Class argument expected.",caller)
        end
        nest =( mod.name.empty? ? mod.class_eval('Module.nesting').last.name : mod.name )
        unless nest =~ /(\AAuthor)/ # begins with 'Author'
          raise(NotImplementedError,"For use within the ::Author namespace only!",caller)
        else
          if mod.name.empty? && mod.class==(Class)
            # It is a Proxy class, so OK to include THIS module:
            super 
            #
          else # It is NOT a Proxy class
            if mod.class==(Class)
              # Use this module to extend the class, instead of including it:
              file, line = caller[0].split(':')
              mod.class_eval( %[extend(#{self})], file, line.to_i )
            elsif mod.class==(Module)
              # Include this module within a proxy class of the module:
              file, line = caller[0].split(':')
              mod.module_eval( %[
                class << self
                  include(#{self})
                end
              ], file, line.to_i )
            end
          end
        end
      end

      def self.extend_object(mod)
        unless mod.class==(Class) || mod.class==(Module)
          # Do not allow instance objects to be extended, with this module.
          raise(TypeError,"Module or Class argument expected.",caller)
        end
        if mod.class==(Module) # include it instead
          file, line = caller[0].split(':')
          mod.module_eval( %[
            class << self
              include(#{self})
            end
          ], file, line.to_i )
        elsif (mod.name.empty? && mod.class==(Class)) # include it instead
          file, line = caller[0].split(':')
          mod.class_eval( %[include(#{self})], file, line.to_i )
        else
          nest =( mod.name.empty? ? mod.class_eval('Module.nesting').last.name : mod.name )
          unless nest =~ /(\AAuthor)/ # begins with 'Author'
            raise(NotImplementedError,"For use within the ::Author namespace only!",caller)
          else
            super # OK to extend the object with THIS module.
          end
        end
      end

      def self.extended(mod)
        @@included << mod unless @@included.include?(mod)
      end

      def self.included(mod)
        @@included << mod unless @@included.include?(mod)
      end
    #
    #}#
    
    #{#  PUBLIC CLASS/MODULE METHODS
    #
      public

      def self.included?(mod)
        @@included.included?(mod)
      end

      def self.list_included()
        puts(@@included)
      end

      def self.list_loaded_files()
        puts(@@loaded_files)
      end
    #
    #}#

    #{#  PRIVATE MIXIN METHODS
    #
      private

      def file_loaded(arg)
        unless file_loaded?(arg)
          @@loaded_files.push(arg.to_s)
          return true
        else
          return false
        end
      end
    #
    #}#
    
    #{#  PUBLIC MIXIN METHODS
    #
      public

      def file_loaded?(arg)
        @@loaded_files.include?(arg.to_s)
      end

      def list_loaded_files()
        puts(@@loaded_files)
      end

      def loaded_files()
        @@loaded_files.dup
      end 
    #
    #}#
    
  end # mixin module LoadUtil


  #  Mixin the LoadUtil module, into Author toplevel namespace:
  class << self
    include(LoadUtil)
  end

  file_loaded('Mixin Module - Author::LoadUtil')

end # module Author
